/******************************************************************************
 CMultStdPopupPane.c for TCL 1.1.3 or later

		A subclass of CStdPopupPane that allows for the same pop-up menu to be 
		used twice independently at the same time, either within one window or 
		in separate windows.
		
		When you have one pop-up menu that is used more than once within one
		window, you must create a separate pseudo-menu id for each instance
		of the pop-up menu pane. This menu id should not correspond to the 
		menu id of any menu resource your application uses. This class 
		tricks the TCL into using the pseudo-menu id for the menu; it is
		this mechanism which allows several instances of the same menu to
		function independently.
		
		If you use this class, your program must not have any menus whose
		menu id is 5368 or 5369. Deactivated menus ids are set to one of
		these values (which I picked completely randomly). This prevents
		the TCL from changing all instances of a pop-up menu pane that
		appears in several different windows when the settings on a single
		such pane are changed.
		
		If you create an instance of this class in a window that is already
		activated, you must activate the menu manually so that it works 
		correctly.
		
		This class relies on the inner workings of the CBartender class.
		Changes in CBartender in v. 1.1.3 of the TCL required a major
		reworking of this class. Thus, this class will NOT work with
		v. 1.1.2 or earlier of the TCL. (The problem is related to how
		the TCL deals with an attempt to add the same menu twice to
		the bartender.)
		
		It would be easy to modify this class to work with arrow-pop-up
		panes rather than standard ones.
		
	SUPERCLASS = CStdPopupPane
	
	Copyright  1993 Michael Abramowicz. All rights reserved.
	

 ******************************************************************************/

#include "CMultStdPopupPane.h"
#include "CBartender.h"
									
extern CBartender	*gBartender;

/******************************************************************************
 IMultStdPopupPane
 
 	Create a standard popup menu pane. Works just like IStdPopupPane, but
 		Pass to pseudoMenuID the menu id you would like to use for the
 		pop-up menu. This should equal menuID if and only if the pop-up pane
 		appears no more than once within any given window. Otherwise,
 		pseudoMenuID should be set to some other value not used by any other
 		menu. For example, you might use 4000, 4001, and 4002 as pseudo
 		IDs for three instances of the same pop-up.
******************************************************************************/

void CMultStdPopupPane::IMultStdPopupPane( short menuID, short pseudoMenuID, 
				CView *anEnclosure, CBureaucrat *aSupervisor, 
				short aWidth, short aHeight, short aHEncl, short aVEncl)
{
	short	oldIndex;		/* bartender index number of menu with 		 	*/
							/* same pseudoMenuID							*/

		// Prevent bartender confusion by ensuring there is no other
		// menu enabled with the same pseudo menu id
		
	oldIndex = gBartender->FindMenuIndex(pseudoMenuID);
	if (oldIndex != -1) /* Check for a menu already open with this id */
		ChangeMenuID(oldIndex, kDisabledMenuID);
						/* Temporarily disable that menu */

		// Call a separate routine to do the work of adding the menu
		// to the bartenders list.
		
	IMultStdPopupPaneX(menuID, pseudoMenuID);

		// Do other initialization for a standard popup pane. The
		// menu will now have been added twice, so we must remove
		// one addition.
	CPopupPane::IPopupPane( pseudoMenuID, TRUE, FALSE, anEnclosure, 
			aSupervisor, aWidth, aHeight, aHEncl, aVEncl,
			sizFIXEDSTICKY, sizFIXEDSTICKY);
	IStdPopupPaneX( TRUE);
	gBartender->RemoveMenu( pseudoMenuID);
	Disable();
	
	// In case a menu was temporarily disabled, reenable it
		
	if (oldIndex != -1)	
		ChangeMenuID(oldIndex, pseudoMenuID);
						/* Restore the menu id of the identical menu
						that is currently active. */
}	/* CStdPopupPane::IStdPopupPane */

/******************************************************************************
 IMultStdPopupPaneX
 
 	Add the appropriate menu to the bartender so that the bartender recognizes
 	it through its pseudo menu id.
******************************************************************************/

void CMultStdPopupPane::IMultStdPopupPaneX( short menuID, short pseudoMenuID)
{
	short	oldIndex,		/* bartender index number of menu with menuID */
			newIndex;		/* bartender index of new menu */
			
		// Prevent bartender confusion by ensuring there is no other
		// menu enabled with the same menu id
		
	oldIndex = gBartender->FindMenuIndex(menuID);
	if (oldIndex != -1) /* Check for a menu already open with this id */
		ChangeMenuID(oldIndex, kDisabledMenuID);
						/* Temporarily disable that menu */
	
		// Add the menu to the bartender.
		
	gBartender->AddMenu(menuID, FALSE, 0);
						/* Do not install menu in the menu list. */
	itsMacMenu = gBartender->FindMacMenu(menuID);
						/* Store a handle to the Toolbox menu record
						in an instance variable */
	DetachResource((Handle) itsMacMenu);
						/* Detach the menu resource from the resource fork
						so that additional copies of the Toolbox menu
						record can be kept for other menus. */
	
		// Change the menus ID to the pseudo ID
		
	newIndex = FindItsMenuIndex();
						/* Locate menu index based on handle to Toolbox 
						menu record. */

	ChangeMenuID(newIndex, pseudoMenuID);
						/* Change the bartenders information on the
						menu ID as well as the ID stored within the
						Toolbox menu record. */
	
		// In case a menu was temporarily disabled, reenable it
		
	if (oldIndex != -1)	
		ChangeMenuID(oldIndex, menuID);
						/* Restore the menu id of the identical menu
						that is currently active. */

		// Record the pseudo menu id in an instance variable,
		// so that menu can be enabled and disabled properly.
	itsPseudoMenuID = pseudoMenuID;
	
}

/******************************************************************************
 IViewTemp {OVERRIDE}
 
 	Create a standard popup menu pane from a view template
******************************************************************************/

void CMultStdPopupPane::IViewTemp( CView *anEnclosure, CBureaucrat *aSupervisor,
						Ptr	viewData)
{
	MultStdPopupTemp	*data = (MultStdPopupTemp*) viewData;
	short	oldIndex;	/* bartender index number of menu with 		 	*/
						/* same pseudoMenuID							*/
	short 	menuID		 = data->stdPopupTemp.popupTemp.menuID,
			pseudoMenuID = data->pseudoMenuID;

		// Prevent bartender confusion by ensuring there is no other
		// menu enabled with the same pseudo menu id
		
	oldIndex = gBartender->FindMenuIndex(pseudoMenuID);
	if (oldIndex != -1) /* Check for a menu already open with this id */
		ChangeMenuID(oldIndex, kDisabledMenuID);
						/* Temporarily disable that menu */	

		// Add the new menu to the bartender.

	IMultStdPopupPaneX( menuID, pseudoMenuID);

		// Initialize the pop-up menu class, skipping over
		// the immediate super-class so that the hook is
		// installed correctly.
		
	data->stdPopupTemp.popupTemp.menuID = itsPseudoMenuID;
	if (data->stdPopupTemp.hAutosizing)
		data->stdPopupTemp.popupTemp.paneTemp.width = kAutoSize;
	if (data->stdPopupTemp.vAutosizing)
		data->stdPopupTemp.popupTemp.paneTemp.height = kAutoSize;
	
		// Do other initialization for a standard popup pane. 
		// Note that we trick the other initialization routines
		// into using the pseudoMenuID by directly modifying
		// the template data.
	data->stdPopupTemp.popupTemp.menuID = pseudoMenuID;
	CPopupPane::IViewTemp( anEnclosure, aSupervisor, viewData);
	IStdPopupPaneX( TRUE);
		// The menu will now have been added twice, so we must remove
		// one addition.
	gBartender->RemoveMenu( pseudoMenuID);
	Disable();
	
		// In case a menu was temporarily disabled, reenable it
		
	if (oldIndex != -1)	
		ChangeMenuID(oldIndex, pseudoMenuID);
						/* Restore the menu id of the identical menu
						that is currently active. */

}	/* CMultStdPopupPane::IViewTemp */

/******************************************************************************
 Activate
 
 	Enable the menu before activating it so that the bartender can recognize
 	it again.
******************************************************************************/

void CMultStdPopupPane::Activate( void)
{
	Enable();
	inherited::Activate();
}	/* CPopupPane::Activate */

/******************************************************************************
 Deactivate

	Disable the menu after deactivating it, so that if the bartender tries
	to recognize another menu of the same type, it does not get confused.
******************************************************************************/

void CMultStdPopupPane::Deactivate( void)
{
	inherited::Deactivate();
	Disable();
}	/* CPopupPane::Deactivate */

/******************************************************************************
 Dispose

	Dispose of the detached Toolbox menu record when we are all done with
	it.
******************************************************************************/

void CMultStdPopupPane::Dispose( void)
{
	short	oldIndex;		/* bartender index number of menu with 		 	*/
							/* same pseudoMenuID							*/
		
		// We need to make sure that the menu is not disabled when 
		// it is being disposed, and that there are no other menus
		// that have the same pseudo-id active at the time of disposal.
		// Thus we change the menu to a unique temporary id, look for
		// other menus with the pseudo id and change them temporarily,
		// and then set the menu back to its pseudo-id.
							
	ChangeMenuID(FindItsMenuIndex(), kTemporaryMenuID);
					/* Give it a unique id */
	oldIndex = gBartender->FindMenuIndex(itsPseudoMenuID);
	if (oldIndex != -1) /* Check for a menu already open with this id */
		ChangeMenuID(oldIndex, kDisabledMenuID);
						/* Temporarily disable that menu */
	ChangeMenuID(FindItsMenuIndex(), itsPseudoMenuID);
					
		// We must dispose of the space we allocated for the menu hook.
		// Since we called DetachResource on the Toolbox menu record, the
		// Menu Manager will not do it automatically for us.
		
	DisposeHandle( (Handle) (**itsMacMenu).menuProc);
		
		// Dispose of the pop-up pane and its menu.

	inherited::Dispose();
	
		// If another menus id was changed, restore the id.
		
	if (oldIndex != -1)
		ChangeMenuID(oldIndex, itsPseudoMenuID);
}

/**** I N T E R N A L   M E T H O D S ****/

/******************************************************************************
 Enable

	Set the menu id to the correct menu id (the pseudo menu id) so that 
	the bartender can once again access this menu.
******************************************************************************/

void CMultStdPopupPane::Enable( void)
{
	ChangeMenuID(FindItsMenuIndex(), itsPseudoMenuID);
}

/******************************************************************************
 Disable

	Set the menu id to a menu id not used by other menus so that the bartender
	can no longer access this menu.
******************************************************************************/

void CMultStdPopupPane::Disable( void)
{
	ChangeMenuID(FindItsMenuIndex(), kDisabledMenuID);
}

/******************************************************************************
 ChangeMenuID

	Given an index within the bartenders table, set a new menu ID,
	from both the bartenders and the Toolboxs perspectives.
******************************************************************************/

void CMultStdPopupPane::ChangeMenuID(short index, short newID)
{
	MenuEntryH		menus = gBartender->theMenus;
	MenuHandle		theMacMenu;
	
	if (index != -1) {
		(*menus)[index].MENUid = newID;
		theMacMenu = (*menus)[index].macMenu;
		if (theMacMenu && *theMacMenu)
			(**theMacMenu).menuID = newID;
	}
}

/******************************************************************************
 FindItsMenuIndex

	Looks through the bartenders table for an index entry that matches
	the Toolbox menu record stored in the itsMacMenu instance variable.
	It returns the index number of this entry.
	
	Return -1 if itsMacMenu is not found
******************************************************************************/

short CMultStdPopupPane::FindItsMenuIndex(void)
{
	MenuEntryH			menus = gBartender->theMenus;
										/* The menu table					*/
	short				numMenus = gBartender->numMenus;
										/* The number of menus				*/									
	MenuEntryP			theEntry;		/* Pointer to menu table			*/
	register short		i;				/* Index into menu table			*/
	
	theEntry = *menus;					/* Dereference for speed			*/

	for (i = 0; i < numMenus; i++) {	/* Search thru all entries			*/
		if (theEntry[i].macMenu == itsMacMenu) {
			return(i);					/* Match found. Exit here			*/
		}
	}

										/* We've fallen through the loop	*/
	return(-1);							/*   Menu is not in our table		*/

}